from unittest import TestCase, main
from unittest.mock import Mock

from sagecreator import Configurator
from sagecreator import Provisioner

import os


class ProvisionerTestCase(TestCase):
    _configurator = None

    def setUp(self):
        self._configurator = Configurator()

    def test_provision_should_fail_if_aws_secret_undefined(self):
        self._configurator.get_properties = Mock(return_value={"aws_access_key": "abc"})
        prov = Provisioner(self._configurator)

        with self.assertRaises(ValueError) as context:
            prov.provision("service", "t3.small", 0.2, 1)
            self.assertTrue("Property aws_secret_key is undefined" in str(context))

    def test_provision_should_call_script(self):
        mock_props = {"aws_access_key": "valid",
                      "aws_secret_key": b'dGhlX2F3c19zZWNyZXRfa2V5',
                      "default_private_key_file_name": "default.pem"}
        self._configurator.get_properties = Mock(return_value=mock_props)
        current_env = os.environ.copy()
        current_env["AWS_ACCESS_KEY_ID"] = "valid"
        current_env["AWS_SECRET_ACCESS_KEY"] = "the_aws_secret_key"

        prov = Provisioner(self._configurator)
        prov._call_bootstrap_script = Mock(return_value=None)
        prov.provision("service", "t3.small", 0.2, 1)
        mock_props.update({"service": "service", "instance_type": "t3.small", "spot_price": 0.2, "cluster_size": 1})
        prov._call_bootstrap_script.assert_called_once_with(mock_props, "{}/.ssh/{}".format(self._configurator.get_root_path(), "default.pem"),
                                                            current_env)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

EPHEMERAL_STORAGE_DEVICES = {
    'c1.medium': 1,
    'c1.xlarge': 4,
    'c3.large': 2,
    'c3.xlarge': 2,
    'c3.2xlarge': 2,
    'c3.4xlarge': 2,
    'c3.8xlarge': 2,
    'cc2.8xlarge': 4,
    'cg1.4xlarge': 2,
    'cr1.8xlarge': 2,
    'd2.xlarge': 3,
    'd2.2xlarge': 6,
    'd2.4xlarge': 12,
    'd2.8xlarge': 24,
    'g2.2xlarge': 1,
    'g2.8xlarge': 2,
    'hi1.4xlarge': 2,
    'hs1.8xlarge': 24,
    'i2.xlarge': 1,
    'i2.2xlarge': 2,
    'i2.4xlarge': 4,
    'i2.8xlarge': 8,
    'm1.small': 1,
    'm1.medium': 2,
    'm1.large': 2,
    'm1.xlarge': 4,
    'm2.xlarge': 1,
    'm2.2xlarge': 1,
    'm2.4xlarge': 2,
    'm3.medium': 1,
    'm3.large': 1,
    'm3.xlarge': 2,
    'm3.2xlarge': 2,
    'r3.large': 1,
    'r3.xlarge': 1,
    'r3.2xlarge': 1,
    'r3.4xlarge': 1,
    'r3.8xlarge': 2
}

DEVICE_SEQUENCE = 'bcdefghijklmnopqrstuvwxy'


class Application(object):
    def __init__(self, **kwargs):
        # Set all given parameters
        for key, val in kwargs.items():
            setattr(self, key, val)

    def get_ephemeral_block_mapping(self):
        bdm = {}
        device_map = {k: v for k, v in EPHEMERAL_STORAGE_DEVICES.items() if v > 1}

        if self.instance_type in device_map:
            for i in range(0, device_map[self.instance_type]):
                device = {'ephemeral_name': "ephemeral{0}".format(i)}
                bdm['/dev/sd{0}'.format(DEVICE_SEQUENCE[i])] = device
        return bdm

    def get_block_mapping_ansible(self):
        bdm = list()
        root_device = {
            'volume_type': self.root_volume_type,
            'volume_size': self.root_volume_size,
            'delete_on_termination': self.ebs_delete_on_termination
        }

        if self.os_type == 'debian':
            root_device['device_name'] = '/dev/xvda'
        elif self.os_type == 'ubuntu' or self.os_type == 'bionic':
            root_device['device_name'] = '/dev/sda1'

        bdm.append(root_device)
        for (k, v) in self.get_ephemeral_block_mapping().items():
            d = dict(device_name=k, ephemeral=v['ephemeral_name'])
            bdm.append(d)

        if self.ebs_create_volumes:
            device_count = len(bdm) - 1  # For zero-based index
            for vol_count in range(0, self.ebs_volume_count):
                d = dict(device_name=self.device_name(DEVICE_SEQUENCE[device_count]))
                d['volume_type'] = self.ebs_volume_type
                if self.ebs_volume_iops:
                    d['iops'] = self.ebs_volume_iops

                if self.ebs_volume_size:
                    d['volume_size'] = self.ebs_volume_size

                d['delete_on_termination'] = self.ebs_delete_on_termination
                device_count = device_count + 1
                bdm.append(d)

        return bdm

    def device_name(self, sequence_combination):
        if self.os_type == 'debian':
            return '/dev/xvd{}'.format(sequence_combination)
        elif self.os_type == 'ubuntu' or self.os_type == 'bionic':
            return '/dev/sd{}'.format(sequence_combination)

        raise Exception("Unknown OS type")


def run(argument_spec, argv):
    print(argv)
    from argparse import ArgumentParser

    ap = ArgumentParser(prog="block_device_mapping")

    for k, v in argument_spec.items():
        ap.add_argument("--{}".format(k), required=v['required'])

    args = ap.parse_args(argv)
    app = Application(instance_type=args.instance_type)
    print(json.dumps(app.get_ephemeral_block_mapping(), sort_keys=True, indent=4))
    print(json.dumps(app.get_block_mapping_ansible(), sort_keys=True, indent=4))


def ansible_run(argument_spec):
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False,
    )

    try:
        app = Application(
            module=module,
            **module.params
        )
        module.exit_json(changed=False, result=app.get_block_mapping_ansible())

    except Exception as e:
        module.fail_json(msg=str(e))


from ansible.module_utils.basic import *


def main():
    argument_spec = dict(
        instance_type=dict(required=True, type='str'),
        os_type=dict(required=True, type='str', choices=['debian', 'ubuntu', 'bionic']),
        root_volume_size=dict(required=False, type='str'),
        root_volume_type=dict(required=False, type='str', choices=['gp2', 'io1', 'standard'], default='standard'),
        ebs_create_volumes=dict(required=False, type='bool'),
        ebs_volume_count=dict(required=False, type='int'),
        ebs_volume_type=dict(required=False, type='str', choices=['gp2', 'io1', 'standard'], default='standard'),
        ebs_volume_size=dict(required=False, type='int'),
        ebs_volume_iops=dict(required=False, type='int'),
        ebs_delete_on_termination=dict(required=False, type='bool', default=True),
    )
    ansible_run(argument_spec)


if __name__ == '__main__':
    main()



from ansible.plugins.lookup import LookupBase
import boto.ec2


class LookupModule(LookupBase):

    def __init__(self, loader=None, templar=None, **kwargs):
        super(LookupModule, self).__init__(loader, templar, **kwargs)
        self.conn = boto.ec2.connect_to_region('us-east-1')

    def run(self, terms, variables=None, **kwargs):
        env = kwargs.get("env")
        owner = kwargs.get("owner")
        service = kwargs.get("service")
        reservations = self.conn.get_all_instances(filters={"tag:Environment": env, "tag:Owner": owner, "tag:Service": service})
        ret_val = []
        for r in reservations:
            for instance in r.instances:
                if instance.state == "running" or instance.state == "pending":
                    ret_val.append({"ip_address": instance.ip_address, "private_ip_address": instance.private_ip_address})

        return ret_val



# !/usr/bin/python

import os
import subprocess
import base64


class Provisioner:

    def __init__(self, configurator):
        self._configurator = configurator

    def provision(self, service, instance_type, spot_price, cluster_size):
        props = self._configurator.get_properties()
        props.update({'service': service,
                      'instance_type': instance_type,
                      'spot_price': spot_price,
                      'cluster_size': cluster_size})
        private_key_file = self._get_private_key_file(props)
        current_env = self._get_env(props)
        self._call_bootstrap_script(props, private_key_file, current_env)

    def _call_bootstrap_script(self, props, private_key_file, current_env):
        rc = subprocess.call(
            ["{}/bootstrap.sh".format(self._configurator.get_root_path()),
             "--{}={}".format('private_key_file', private_key_file),
             "--{}={}".format('service', props.get('service')),
             "--{}={}".format('instance_type', props.get('instance_type')),
             "--{}={}".format('spot_price', props.get('spot_price')),
             "--{}={}".format('cluster_size', int(props.get('cluster_size')))], env=current_env)
        return rc

    def terminate(self, service):
        current_env = self._get_env(self._configurator.get_properties())
        rc = subprocess.call(["{}/terminate.sh".format(self._configurator.get_root_path()),
                              "--{}={}".format('service', service)], env=current_env)
        return rc

    def _get_env(self, props):
        current_env = os.environ.copy()
        current_env["AWS_ACCESS_KEY_ID"] = self.validate_and_get("aws_access_key", props)
        current_env["AWS_SECRET_ACCESS_KEY"] = base64.b64decode(self.validate_and_get("aws_secret_key", props)).decode("utf-8")
        return current_env

    def _get_private_key_file(self, props):
        if "private_key_file" in props:
            return props.get("private_key_file")
        return "{}/.ssh/{}".format(self._configurator.get_root_path(), self.validate_and_get("default_private_key_file_name", props))

    @staticmethod
    def validate_and_get(prop_name, props):
        if prop_name not in props:
            raise ValueError("Property {} is undefined".format(prop_name))
        return props.get(prop_name)



import logging

import click

from sagecreator.configurator import Configurator
from sagecreator.provisioner import Provisioner

logging.basicConfig(level=logging.INFO, format='%(name)-12s: %(levelname)-8s: %(message)s')
log = logging.getLogger(__name__)

configurator = Configurator()


@click.group()
def cli():
    pass


def valid_instance_type(ctx, param, value):
    # valid_types = configurator.get_valid_instance_types()
    # if value not in valid_types:
    #     raise click.BadParameter("Invalid instance type {}. Should be one of {}".format(value, str(valid_types)))
    return value


@cli.command(help="Configure infrastructure settings")
@click.option('--access_key_id', prompt=True, required=True)
@click.option('--secret_access_key', prompt=True, required=True, hide_input=True, confirmation_prompt=True)
@click.option('--company', prompt=True, required=True, help="Company name")
@click.option('--owner', prompt=True, required=True, help="Owner / Team that owns the service")
@click.option('--key_pair_name', prompt='Key pair name (optional, if not provided it will be created with a new private key)', default='',
              help="Provide existing key pair name")
def configure(access_key_id, secret_access_key, company, owner, key_pair_name):
    try:
        private_key_file = None
        if key_pair_name:
            private_key_file = click.prompt("Private key file (required because Key pair name was provided)", default='')
            if not private_key_file:
                raise click.BadParameter("Private key file has to be provided because Key pair name was provided")
        configurator.persist(access_key_id, secret_access_key, company, owner, private_key_file, key_pair_name)
    except Exception as e:
        log.error("Failed to store configuration. {}".format(str(e)))


@cli.command(help="Provision instances")
@click.option('--service', prompt=True, required=True, help="Name of the service")
@click.option('--instance_type', prompt=True, default="t3.small", help="AWS instance type (default t3.small)", callback=valid_instance_type)
@click.option('--spot_price', prompt='Spot instance price', default=0.1, help="Price of the spot instance (default $0.1)")
@click.option('--cluster_size', prompt=True, default=1, type=int, help="Size of the cluster (default 1)")
def provision(service, instance_type, spot_price, cluster_size):
    try:
        click.echo("Provisioning cluster based on {}".format(configurator.get_config_path()))
        prov = Provisioner(configurator)
        prov.provision(service, instance_type, spot_price, cluster_size)
    except Exception as e:
        log.error("Failed to provision instance(s). {}".format(str(e)))


@cli.command(help="Terminate instance(s)")
@click.option('--service', prompt=True, required=True, help="Name of the service")
@click.confirmation_option(prompt='Are you sure you want to terminate the cluster?')
def terminate(service):
    prov = Provisioner(configurator)
    prov.terminate(service)


@cli.command(help="Full path to configuration file")
def pwd():
    click.echo(configurator.get_config_path())


if __name__ == "__main__":
    cli()
